%% IOF 115 - Modelagem Numerica em Oceanografia
% 2a Lista de Exercicios
% Aluno:   Danilo Rodrigues Vieira
% No. USP: 5653262

% Modelo hidrodinamico 2D
% Local: Baia da Guanabara - RJ (43 00' W - 43 20' W e 22 57' S - 22 40' S)

clear; close all; clc;

%Configuracao da grade (pontos tipo eta, u, v)
%NORTE (k)
%    V V V V V
%5  UEUEUEUEUE
%    V V V V V
%3  UEUEUEUEUE
%    V V V V V
%2  UEUEUEUEUE
%    V V V V V
%1  UEUEUEUEUE
%    1 2 3 4 5 LESTE (j)

% desconsiderar primeira coluna e ultima linha da grade acima
% de modo a ter ponto tipo eta em todo o contorno

% Sistema Internacional de unidades (SI)

opt = [1 1];
hperg = figure('color', 'w', 'Colormap', flipud(fliplr(hot(256))), ...
    'NumberTitle', 'off', 'Name', 'Campo de velocidade', ...
    'Units', 'Normalized', 'Position', [0.3 0.3 0.4 0.4], ...
    'MenuBar', 'none', 'Toolbar', 'none');
b1 = uicontrol(hperg, 'Style', 'pushbutton', 'String', 'Vento de SW + oscilacao senoidal', ...
    'Units', 'Normalized', 'Position', [0.1 0.1 0.8 0.35], ...
    'Callback', 'opt=[1 1];close(hperg);');
b2 = uicontrol(hperg, 'Style', 'pushbutton', 'String', 'Vento de SW', ...
    'Units', 'Normalized', 'Position', [0.1 0.55 0.35 0.35], ...
    'Callback', 'opt=[0 1];u=0;close(hperg);');
b3 = uicontrol(hperg, 'Style', 'pushbutton', 'String', 'Oscilacao senoidal', ...
    'Units', 'Normalized', 'Position', [0.55 0.55 0.35 0.35], ...
    'Callback', 'opt=[1 0];close(hperg);');
uiwait(hperg);


% Area abrangida pelo modelo
latmin = -22.9500; % 22 57'
latmax = -22.6667; % 22 40'
lonmin = -43.3333; % 43 20'
lonmax = -43.0000; % 43 00'
load costa3.mat; % variaveis: costa | batimetria

% Quantidade de pontos de grade
dx   = 1000;
kmax = round((abs(latmin - latmax) * 60 * 1852) / dx); % pontos de grade em Y
jmax = round((abs(lonmin - lonmax) * 60 * 1852 * ...   % pontos de grade em X
    cosd(mean([latmin latmax])))/dx);
jmax2 = jmax*2;
kmax2 = kmax*2;

% batimetria (chaves para pontos maritimos)
bat   = batimetria;
bate  = bat(1:2:end-1, 2:2:end);
batu  = bat(1:2:end-1, 1:2:end-1);
batv  = bat(2:2:end  , 2:2:end);
kmare = bate*0;
kmare(bate>0) = 1;
kmaru = batu*0;
kmaru(batu>0) = 1;
kmarv = batv*0;
kmarv(batv>0) = 1;
    
%plotando batimetria
figure('units','normalized','position',[0,0,1,1]);

subplot(1,2,1);
[LON LAT]   = meshgrid(linspace(lonmin,lonmax,jmax),linspace(latmin,latmax,kmax));
[LON2 LAT2] = meshgrid(linspace(lonmin,lonmax,jmax2),linspace(latmin,latmax,kmax2));
pcolor(LON2, LAT2, batimetria); shading flat; caxis([0 max(batimetria(:))]);
hold on
plot(costa(:,1),costa(:,2), '-k');
hold off
title 'Batimetria da regiao modelada (m)';
axis('equal', [lonmin lonmax latmin latmax]);
colorbar vert;

% Constantes iniciais
g     = 9.8;   % aceleracao da gravidade
nmax  = 1000;  % niveis de tempo
dx    = 1000;  % espacamento da grade em x
dy    = 1000;  % espacamento da grade em y
if opt(1), dt = 40; else dt = 10; end    % passo de tempo
rfric = 0.02;  % coeficiente de friccao no fundo
freqplot = 20; % frequencia de plotagem
dens  = 1024;  % densidade media da agua do mar
latid = 25*pi/180;             % latitude (em graus, para rad)
fco   = 2*7.292E-5*sin(latid); % parametro de Coriolis

dens2   = dens*2;
denbatu = dens.*batu;
denbatv = dens.*batv;

coef1 = 1-rfric*dt/2;
coef2 = 1+rfric*dt/2;

% Condicoes iniciais de repouso (1 - valores atuais, 2 - renovados)
eta1 = zeros(kmax, jmax);
u1   = zeros(kmax, jmax);
v1   = zeros(kmax, jmax);
eta2 = zeros(kmax, jmax);
u2   = zeros(kmax, jmax);
v2   = zeros(kmax, jmax);

% Condicoes de vento e calculo das tensoes de cisalhamento na superficie
dens_ar = 1.025;
fric    = 2.6*1E-3;

uwind(1:kmax,1:jmax) = 7.0711 * opt(2);
vwind(1:kmax,1:jmax) = 7.0711 * opt(2);
wwind = sqrt(uwind.^2+vwind.^2);
taux  = fric*dens_ar.*uwind.*wwind;
tauy  = fric*dens_ar.*vwind.*wwind;

vento    = sqrt(uwind.^2+vwind.^2);
ventomax = max(vento(:));

% plotando os ventos
subplot(1,2,2)
hold on
plot(costa(:,1),costa(:,2), '-k');
quiver(LON,LAT,uwind,vwind);
axis('equal', [lonmin lonmax latmin latmax]);
hold off
title(['Vento - intensidade maxima ', num2str(ventomax), ' m/s'])

amplit = 0.25;
omega  = 2*pi/43200;

% Loop no tempo/home/danilo
hf_main = figure('units', 'normalized', 'position', [0,0,1,1]);
for n=1:nmax
    tempo=n*dt;
    
    if opt(1)
        eta1(2, :) = amplit*cos(omega*tempo);
    end
    
    %Eq da Continuidade
    for j=2:jmax-1
        for k=2:kmax-1
           if kmare(k,j)>0
               forcx=(batu(k,j+1).*u1(k,j+1)-batu(k,j).*u1(k,j))/dx;
               forcy=(batv(k,j).*v1(k,j)-batv(k-1,j).*v1(k-1,j))/dy;
               eta2(k,j)=eta1(k,j)-dt*(forcx+forcy);
           end
        end
    end
 
    %Eq. do movimento em x
    for j=2:jmax
        for k=2:kmax-1
            if (kmaru(k,j)*kmare(k,j)*kmare(k,j-1))>0
                vmedu=(v1(k,j)+v1(k,j-1)+v1(k-1,j-1)+v1(k-1,j))/4;
                forc=fco*vmedu-g.*(eta2(k,j)-eta2(k,j-1))./dx...
                    +taux(k,j)./denbatu(k,j);
                u2(k,j)=(coef1*u1(k,j)+forc*dt)/coef2;
            end
         end
    end
    
    %Eq. do movimento em y
    for j=2:jmax-1
        for k=1:kmax-1
            if (kmarv(k,j)*kmare(k,j)*kmare(k+1,j))>0
                umedv=(u2(k,j)+u2(k+1,j)+u2(k+1,j+1)+u2(k,j+1))/4;
                forc=-fco*umedv-g.*(eta2(k+1,j)-eta2(k,j))./dy...
                    +tauy(k,j)./denbatv(k,j);
                v2(k,j)=(coef1.*v1(k,j)+forc*dt)/coef2;
            end
        end
    end
    
    % Renovando as variaveis no tempo
    eta1=eta2;
    u1=u2;
    v1=v2;

    % Plotagem de resultados     
    if ~mod(n,freqplot)
        % Condicoes de contorno nao gradiente
        for j=1:jmax
            eta2(1,j)=(2*eta2(2,j)-eta2(3,j))*kmare(1,j);
            eta2(kmax,j)=(2*eta2(kmax-1,j)-eta2(kmax-2,j))*kmare(kmax,j);
            u2(1,j)=(2*u2(2,j)-u2(3,j))*kmaru(1,j);
            u2(kmax,j)=(2*u2(kmax-1,j)-u2(kmax-2,j))*kmaru(kmax,j);
            v2(kmax,j)=(2*v2(kmax-1,j)-v2(kmax-2,j))*kmarv(kmax,j);
        end
        for k=1:kmax
            eta2(k,1)=(2*eta2(k,2)-eta2(k,3))*kmare(k,1);
            eta2(k,jmax)=(2*eta2(k,jmax-1)-eta2(k,jmax-2))*kmare(k,jmax);
            u2(k,1)=(2*u2(k,2)-u2(k,3))*kmaru(k,1);
            v2(k,1)=(2*v2(k,2)-v2(k,3))*kmaru(k,1);
            v2(k,jmax)=(2*v2(k,jmax-1)-v2(k,jmax-2))*kmarv(k,jmax);
        end

        %Plotando elevacoes e correntes
        velo    = sqrt(u2.^2+v2.^2);
        velomax = max(velo(:));
        etamax  = max(eta2(:));
        etamin  = min(eta2(:));

        figure(hf_main);
        subplot(1,2,1);
        pcolor(LON, LAT, eta2); shading flat;
        colormap jet;
        hold on
        plot(costa(:,1),costa(:,2), '-k');
        hold off;
        colorbar;
        title(sprintf('Elevacoes (m) - tempo %.0f minutos. Limites %.2g a %.2g m',...
            tempo/60, etamax, etamin));
        xlabel 'Longitude';
        ylabel 'Latitude';
        axis('equal', [lonmin lonmax latmin latmax]);
        if opt(1), caxis([-0.5 0.5]); else caxis([-0.02 0.02]); end
        
        subplot(1,2,2);
        quiver(LON,LAT,u2,v2);
        hold on;
        plot(costa(:,1),costa(:,2), '-k');
        hold off
        title(['Correntes (m/s) - intensidade maxima ' num2str(velomax,'%.2g') ' m/s']);
        axis('equal', [lonmin lonmax latmin latmax])
        xlabel 'Longitude';
        ylabel 'Latitude';
        pause(1.25);
    end
end